/*
 * BTLTextStorage.h
 * StoryWriter
 *
 * Handles storywriter specific text storage tasks, such as making sure letters
 * display as capitalized.
 * 
 * Created by Matthew Christensen on Friday Aug 27 2004
 * Copyright © 2004 Matthew Christensen. All rights reserved.
 */

#import <Cocoa/Cocoa.h>

@class BTLDocumentInfo;
@class BTLTemplate;
@class BTLStyle;
@class BTLTextStorageUndoCommand;
@class BTLCard;
@class BTLUndoManager;
@class BTLLayoutManager;
@class BTLCard;

#define INITIAL_VERSION_ID 1

#define ALL_CAPS_ATTRIBUTE @"BTLAllCapsAttribute"
#define NO_ALL_CAPS_ATTRIBUTE @"BTLNoAllCapsAttribute"

// Section header style attribute
#define SECTION_HEADER_ATTRIBUTE @"SectionHeaderStyle"

#define TEMPORARY_TEXT_ATTRIBUTE @"BTLTemporaryText"

#define ADD_CARD_KEY @"BTLAddCard"
#define REMOVE_CARD_KEY @"BTLRemoveCard"

@interface BTLTextStorage : NSTextStorage 
{
	// This string is our main storage string
	NSMutableAttributedString *attributedString;

	// This string contains the string formatted with 
	// capitalization and other char-level changes
	NSMutableAttributedString *formattedString;

	// Associated document info object
	BTLDocumentInfo *documentInfo;
	BTLTemplate *template;
	
	// Style to use after all other text
	BTLStyle *endStyle;
	
	// Used to hold unicode newline characters
	NSMutableCharacterSet *paragraphChangeCharacters;

	// If true disable autocapitalize for the next character
	bool disableAutocapitalizeTemporarily;
	
	int nesting;
	
	// We need to associate with undo managers so we can undo style changes
	BTLUndoManager *undoManager;
	
	// Our list of cards
	NSMutableArray *cards;
	NSMutableDictionary *cardsById;
	
	// ID of the next created card
	unsigned nextCardId;
	
	// Tracks our version, so other classes can tell if we've changed
	uint32_t versionID;
	
	// Are card notes showing?
	bool notesHidden;
	
	bool disableCardRemove;
	
	bool keepNotesHidden; // If true, keep notes hidden no matter what
	
 	bool updateCardLocations; // If true, cardl ocations are updated on text edit
	
	bool changeNotes; // If true, card notes are edited on text edit
	
	bool updateCards; // If true, cards name/notes are updated on range changes

	bool addCards; // if false cards won't be added
}



#pragma mark Initialization

// Designated initializer
- init;

- initWithDocumentInfo: (BTLDocumentInfo *) info template: (BTLTemplate *) template;

- initWithDocumentInfo: (BTLDocumentInfo *) info template: (BTLTemplate *) template attributedString: (NSAttributedString *) str;

- (void) setTemplate: (BTLTemplate *) t documentInfo: (BTLDocumentInfo *) info;


#pragma mark NSCoding support

- (void)encodeWithCoder:(NSCoder *)encoder;
- (id)initWithCoder:(NSCoder *)decoder;

#pragma mark Undo management

// Call when you want to start a new undo group,
// meaning that everything from this point up to the
// next startNewUndoGroup will be undone
- (void) startNewUndoGroup;

// Run the specified undo command, then release it
- (void) runUndoCommand: (BTLTextStorageUndoCommand *) cmd ;

// Add the specified undo command, optionally following it with a terminator
// with the given action name for the menu
- (void) addUndoCommand: (BTLTextStorageUndoCommand *) cmd action: (NSString *) name;

// Snapshot cards array
- (void) snapshotCardsArray;

// Restore array
- (void) restoreCardsArray: (NSArray *) a;

#pragma mark Utility methods

- (BTLLayoutManager *) layoutManager;

// Adds a page break line at the given location
- (void) insertPageBreak: (unsigned) location;

// Return true if the line at the given location is ap age break
- (bool) lineBreakAtLocation:(unsigned) location;

// Find the location of the next line break after a location
- (unsigned) locationOfLineBreakAfterLocation: (unsigned) location;

// Lowest-level routine for placing text
// Passed the formatted and unformatted strings and a range, adds them
// and sends a notificaton
- (void) storeTextForRange: (NSRange) range
		  attributedString: (NSAttributedString *) as
		   formattedString: fs;

// Called when our linked document info changes
- (void) documentInfoChanged: (NSNotification *) note;

// Insert a string with a given style at a range
- (void) insertStringWithStyle: (NSString *) str 
						 style: (BTLStyle *) style
						 range: (NSRange) range
						  card: (BTLCard *) card;

// Clear all text in the string and reset to default style
- (void) clearText;

// Given a location in our string, return the style for the current paragraph (with the given template)
// The current paragraph will start after the most recent \n char
- (BTLStyle *) styleForParagraph: (unsigned) loc;

- (void) appendString: (NSString *) string;

// Set the style for the paragraph at the given location
- (unsigned ) setStyleForParagraph: (BTLStyle *) style 
						  location: (unsigned) location
							  card: (BTLCard *) card;
- (unsigned ) setStyleForParagraph: (BTLStyle *) style 
						  location: (unsigned) location
				  ignoreEmptyLines: (bool) ignore
					   lineIgnored: (bool *) lineIgnored
							  card: (BTLCard *) card;


// Return the style for a given index
- (BTLStyle *) styleForLocation: (unsigned) location;

// Return the range of the line for a given location
- (NSRange) rangeOfLineForLocation: (unsigned) location;

// Find the effective range for the style at the given location
- (NSRange) rangeForStyleAtLocation: (unsigned) location;

// Insert a string at a location
- (void) insertString: (NSString *)s atIndex: (unsigned) location;

// Disable autocapitalize for the next character
- (void) disableAutocapitalizeTemporarily;

// Return the current line for a given location
- (NSString *) lineForLocation: (unsigned) location;

// Replace the characters in the range range with the string str without having the template process the text. If
- (void)replaceCharactersInRangeRaw:(NSRange)range withAttributedString:(NSAttributedString *)str;
- (void)replaceCharactersInRangeRaw:(NSRange)startRange 
			   withAttributedString:(NSAttributedString *)str
				 incrementLocations: (bool) incrementLocations;

#pragma mark Accessors

- (NSUndoManager *) undoManager;

- (uint32_t) versionID;
- (void) incrementVersionID;

- (void) setKeepNotesHidden: (bool) b;

- (BTLTemplate *) template;
- (BTLDocumentInfo *) documentInfo;

#pragma mark Overrides

- (NSString *) string;

- (void)replaceCharactersInRange:(NSRange)range withString:(NSString *)str;
- (void)replaceCharactersInRange:(NSRange)range withString:(NSString *)str startStyle: (BTLStyle *) s;

- (void)setAttributes:(NSDictionary *)attrs range:(NSRange)range;
- (NSDictionary *)attributesAtIndex:(unsigned)index effectiveRange:(NSRangePointer)aRange;



#pragma mark Card management

- (void) updatePrevCardAfterRemove: (int) i
						  oldRange: (NSRange) crange
					 affectedRange: (NSRange) r;

// This updates cards assuming that the range r will be removed from the text
// It returns the actual range removed. Note that this can be different than the 
// starting range, if notes are removed as well
- (NSRange) updateCardsForRangeRemove: (NSRange) r;

// Given a range removal, update the card for that range
- (void) updateCardForRangeRemoval: (BTLCard *) c range: (NSRange) r;

- (void) updateCardLengths: (BTLCard *) card withDelta: (int) delta;

// manages showing/hiding the card notes
- (void) showCardNotes;
- (void) hideCardNotes;
- (bool) notesHidden;
- (void) showCardNotes: (bool) titlesHidden;
- (void) hideCardNotes: (bool) titlesHidden;

// Post a notification that the cards have changed
- (void) postCardChangedNotification;

// List of cards
- (NSMutableArray *) cards;

// Called when a card changed
- (void) cardChanged: (NSNotification *) card;

// Return the card for a given ID
- (BTLCard *) cardForID: (NSNumber *) cardId;

- (void) addCardRaw: (BTLCard *) card 
	   afterCard: (BTLCard *) afterCard;

// Add a card object.
// If afterCard is nil, adds to beginning. Otherwise inserts after specified card
// If card ID is nil, it will be set to the next id
- (void) addCard: (BTLCard *) card 
	   afterCard: (BTLCard *) afterCard;

// Add a card object.
// If afterCard is nil, adds to beginning. Otherwise inserts after specified card
// If card ID is nil, it will be set to the next id
// If card is linked, location will be used to position it. 
- (void) addCard: (BTLCard *) card 
	   afterCard: (BTLCard *) afterCard
		location: (int) location;

// Add a card object. Used for undo
- (void) removeCard: (BTLCard *) sec;

// Return the index in the list for a given card
- (unsigned) indexForCard: (BTLCard *) s;

// Returns the card closest to this locatation, even if the card is out of range
// If location is before first element, returns nil. If after last element,
// return final card
- (BTLCard *) nearestCardForLocation: (unsigned) location;

// Return the card for a given location, or nil if no such card
- (BTLCard *) cardForLocation: (unsigned) loc;

// Return the card for the line at a given location. If there is no card that starts on that line,
// return nil
- (BTLCard *) cardForLineAtLocation: (unsigned) loc;

// Return the location for a given card.If not found,
// location will equal length
- (unsigned) locationForCard: (BTLCard *) c;

// Move a card
- (void) moveCardAtIndex: (unsigned) fromIndex toIndex: (unsigned) toIndex;

// Move a card
- (void) moveCardAtIndex: (unsigned) fromIndex toIndex: (unsigned) toIndex justTitle: (bool) justTitle;

// Adds a card for the given locaiton, if one does not exist on the line
// already.
- (void) addCardForLocation: (unsigned) location;


// Return the range of a card title and any notes
- (NSRange) rangeForCardTitle: (BTLCard *) c;

	// Return the range for a given card
- (NSRange) rangeForCard: (BTLCard *) c;

// Shows the notes for a given card. Returns the location
// after all the notes
- (unsigned) displayNotesForCard: (BTLCard *) c;



@end

// Posted when one or more cards have been modified by the text storage
NSString *BTLCardsChangedNotification;

